aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorRostislav Skudnov <rostislav@tuxera.com>2018-10-15 10:26:15 +0000
committerDenys Vlasenko <vda.linux@googlemail.com>2018-10-30 16:55:09 +0100
commit3f6a978a0dabe08e9df15b3f9ee2bd3cf8e3c441 (patch)
tree60c1ddc5b1b41b89e70625fab629435465f954aa
parentf3e2838fc4e6bd0713d7ee5a17e752a19870a0f8 (diff)
downloadbusybox-3f6a978a0dabe08e9df15b3f9ee2bd3cf8e3c441.tar.gz
dd: add 'oflag=seek_bytes'
Allow specifying position in the output file in bytes instead of obs-sized blocks, improve compatibility with GNU dd. function old new delta dd_main 1632 1693 +61 packed_usage 33130 33150 +20 static.oflag_words - 12 +12 ------------------------------------------------------------------------------ (add/remove: 1/0 grow/shrink: 2/0 up/down: 93/0) Total: 93 bytes Signed-off-by: Rostislav Skudnov <rostislav@tuxera.com> Signed-off-by: Denys Vlasenko <vda.linux@googlemail.com>
-rw-r--r--coreutils/dd.c33
-rw-r--r--docs/posix_conformance.txt1
2 files changed, 25 insertions, 9 deletions
diff --git a/docs/posix_conformance.txt b/docs/posix_conformance.txt
index cdf89b744..0e6f4a317 100644
--- a/docs/posix_conformance.txt
+++ b/docs/posix_conformance.txt
@@ -182,6 +182,7 @@ dd compatibility options:
conv=fsync | yes | |
iflag=skip_bytes| yes | |
iflag=fullblock | yes | |
+ oflag=seek_bytes| yes | |
df POSIX options
option | exists | compliant | remarks
diff --git a/coreutils/dd.c b/coreutils/dd.c
index 0f130bcad..129637e24 100644
--- a/coreutils/dd.c
+++ b/coreutils/dd.c
@@ -37,7 +37,7 @@
//config: elapsed time and speed.
//config:
//config:config FEATURE_DD_IBS_OBS
//config: bool "Enable ibs, obs, iflag and conv options"
//config: default y
//config: depends on DD
//config: help
@@ -57,7 +57,7 @@
//usage:#define dd_trivial_usage
//usage: "[if=FILE] [of=FILE] " IF_FEATURE_DD_IBS_OBS("[ibs=N] [obs=N] ") "[bs=N] [count=N] [skip=N]\n"
-//usage: " [seek=N]" IF_FEATURE_DD_IBS_OBS(" [conv=notrunc|noerror|sync|fsync] [iflag=skip_bytes|fullblock]")
+//usage: " [seek=N]" IF_FEATURE_DD_IBS_OBS(" [conv=notrunc|noerror|sync|fsync] [iflag=skip_bytes|fullblock] [oflag=seek_bytes]")
//usage:#define dd_full_usage "\n\n"
//usage: "Copy a file with converting and formatting\n"
//usage: "\n if=FILE Read from FILE instead of stdin"
@@ -80,6 +80,7 @@
//usage: "\n conv=swab Swap every pair of bytes"
//usage: "\n iflag=skip_bytes skip=N is in bytes"
//usage: "\n iflag=fullblock Read full blocks"
+//usage: "\n oflag=seek_bytes seek=N is in bytes"
//usage: )
//usage: IF_FEATURE_DD_STATUS(
//usage: "\n status=noxfer Suppress rate output"
@@ -133,10 +134,14 @@ enum {
FLAG_SKIP_BYTES = (1 << 5) * ENABLE_FEATURE_DD_IBS_OBS,
FLAG_FULLBLOCK = (1 << 6) * ENABLE_FEATURE_DD_IBS_OBS,
/* end of input flags */
- FLAG_TWOBUFS = (1 << 7) * ENABLE_FEATURE_DD_IBS_OBS,
- FLAG_COUNT = 1 << 8,
- FLAG_STATUS_NONE = 1 << 9,
- FLAG_STATUS_NOXFER = 1 << 10,
+ /* start of output flags */
+ FLAG_OFLAG_SHIFT = 7,
+ FLAG_SEEK_BYTES = (1 << 7) * ENABLE_FEATURE_DD_IBS_OBS,
+ /* end of output flags */
+ FLAG_TWOBUFS = (1 << 8) * ENABLE_FEATURE_DD_IBS_OBS,
+ FLAG_COUNT = 1 << 9,
+ FLAG_STATUS_NONE = 1 << 10,
+ FLAG_STATUS_NOXFER = 1 << 11,
};
static void dd_output_status(int UNUSED_PARAM cur_signal)
@@ -250,7 +255,7 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
static const char keywords[] ALIGN1 =
"bs\0""count\0""seek\0""skip\0""if\0""of\0"IF_FEATURE_DD_STATUS("status\0")
#if ENABLE_FEATURE_DD_IBS_OBS
- "ibs\0""obs\0""conv\0""iflag\0"
+ "ibs\0""obs\0""conv\0""iflag\0""oflag\0"
#endif
;
#if ENABLE_FEATURE_DD_IBS_OBS
@@ -258,6 +263,8 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
"notrunc\0""sync\0""noerror\0""fsync\0""swab\0";
static const char iflag_words[] ALIGN1 =
"skip_bytes\0""fullblock\0";
+ static const char oflag_words[] ALIGN1 =
+ "seek_bytes\0";
#endif
#if ENABLE_FEATURE_DD_STATUS
static const char status_words[] ALIGN1 =
@@ -276,6 +283,7 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
OP_obs,
OP_conv,
OP_iflag,
+ OP_oflag,
/* Must be in the same order as FLAG_XXX! */
OP_conv_notrunc = 0,
OP_conv_sync,
@@ -297,6 +305,7 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
//swab swap every pair of input bytes: will abort on non-even reads
OP_iflag_skip_bytes,
OP_iflag_fullblock,
+ OP_oflag_seek_bytes,
#endif
};
smallint exitcode = EXIT_FAILURE;
@@ -368,6 +377,10 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
G.flags |= parse_comma_flags(val, iflag_words, "iflag") << FLAG_IFLAG_SHIFT;
/*continue;*/
}
+ if (what == OP_oflag) {
+ G.flags |= parse_comma_flags(val, oflag_words, "oflag") << FLAG_OFLAG_SHIFT;
+ /*continue;*/
+ }
#endif
if (what == OP_bs) {
ibs = xatoul_range_sfx(val, 1, ((size_t)-1L)/2, cwbkMG_suffixes);
@@ -439,7 +452,8 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
xmove_fd(xopen(outfile, oflag), ofd);
if (seek && !(G.flags & FLAG_NOTRUNC)) {
- if (ftruncate(ofd, seek * obs) < 0) {
+ size_t blocksz = (G.flags & FLAG_SEEK_BYTES) ? 1 : obs;
+ if (ftruncate(ofd, seek * blocksz) < 0) {
struct stat st;
if (fstat(ofd, &st) < 0
@@ -472,7 +486,8 @@ int dd_main(int argc UNUSED_PARAM, char **argv)
}
}
if (seek) {
- if (lseek(ofd, seek * obs, SEEK_CUR) < 0)
+ size_t blocksz = (G.flags & FLAG_SEEK_BYTES) ? 1 : obs;
+ if (lseek(ofd, seek * blocksz, SEEK_CUR) < 0)
goto die_outfile;
}
@@ -37,7 +37,7 @@
//config:
//config:config FEATURE_DD_IBS_OBS
-//config: bool "Enable ibs, obs, iflag and conv options"
+//config: bool "Enable ibs, obs, iflag, oflag and conv options"
//config: default y
//config: depends on DD